/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2007,2008 Oracle.  All rights reserved.
 *
 * $Id$
 */

package dbxmltest;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlContainerConfig;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlManager;
import com.sleepycat.dbxml.XmlQueryContext;
import com.sleepycat.dbxml.XmlResults;
import com.sleepycat.dbxml.XmlTransaction;
import com.sleepycat.dbxml.XmlValue;

public class ResolverTest {
    // Container name;
    private static final String CON_NAME = "testData.dbxml";
    private TestConfig hp = null;
    private XmlManager mgr = null;
    private XmlContainer cont = null;

    @BeforeClass
	public static void setupClass() {
	System.out.println("Begin test XmlResolver!");
    }

    @Before
	public void setUp() throws Throwable {
	TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
	hp = new TestConfig(XmlTestRunner.getEnvironmentType(),
			    XmlTestRunner.isNodeContainer(), XmlTestRunner.getEnvironmentPath());
	mgr = hp.createManager();
	cont = hp.createContainer(CON_NAME, mgr);
    }

    @After
	public void tearDown() throws Throwable {
	hp.closeContainer(cont);
	hp.closeManager(mgr);
	hp.closeEnvironment();
	TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
    }

    @AfterClass
	public static void tearDownClass() {
	System.out.println("Finished test XmlResolver!");
    }

    @Test
	public void TestResolveDocument() throws Throwable {
	XmlTransaction txn = null;
	TestResolver test = new TestResolver("stuff.xml", "<root>stuff</root>");
	String queryString = "doc('http://fake.uri.com/stuff.xml')/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults res = null;
	try{
	    mgr.registerResolver(test);
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		res = mgr.query(txn, queryString, context);
	    }else
		res = mgr.query(queryString, context);
	    XmlValue reslt = res.next();
	    assertTrue(reslt.isNode());
	    assertTrue(reslt.asString().equals("<root>stuff</root>"));
	    reslt = new XmlValue();
	    assertTrue(test.resolveDocument(txn, mgr, "stuff.xml", reslt));
	    assertTrue(reslt.isNode());
	    assertTrue(reslt.asString().equals("<root>stuff</root>"));
	}catch(XmlException e){
	    throw e;
	}finally{
	    if(res != null) res.delete();
	    test.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test resolveCollection()
     * 
     * 
     */
    @Test
	public void testResolveCollection() throws Throwable {
	XmlTransaction txn = null;
	TestResolver test = new TestResolver("stuff.xml", "<root>stuff</root>");
	mgr.registerResolver(test);
	String queryString = "collection('http://fake.uri.com/dbxml')/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults res = null;
	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		res = mgr.query(txn, queryString, context);
	    }else
		res = mgr.query(queryString, context);
	    assertEquals(res.size(), 1);
	    XmlValue val = res.next();
	    assertTrue(val.isNode());
	    assertTrue(val.asString().equals("<root>stuff</root>"));
	    res.delete();
	    res = null;
	    res = mgr.createResults();
	    assertTrue(test.resolveCollection(txn, mgr, "stuff.xml", res));
	    assertEquals(res.size(), 1);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    res.delete();
	    test.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test resolveEntity()
     * 
     */
    @Test
	public void testResolveEntity() throws Throwable {
	XmlTransaction txn = null;
	TestResolver test = new TestResolver();
	XmlDocument doc = mgr.createDocument();
	doc.setName("doc1.xml");
	String str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
	    + "<!DOCTYPE root SYSTEM \"http://fake.uri.com"
	    + "/my.dtd\"><root>&stuff;</root>";
	doc.setContent(str);

	try {
	    mgr.registerResolver(test);
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc);
		assertEquals(str, cont.getDocument(txn, "doc1.xml").getContentAsString());
	    }else{
		cont.putDocument(doc);
		assertEquals(str, cont.getDocument("doc1.xml").getContentAsString());
	    }
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(hp.isTransactional() && txn != null) txn.commit();
	    test.delete();
	}
    }

    /*
     * Test resolveSchema()
     * 
     */
    @Test
	public void testResolveSchema() throws Throwable {
	XmlTransaction txn = null;
	TestResolver test = new TestResolver();
	XmlContainerConfig conf = new XmlContainerConfig();
	conf.setAllowValidation(true);
	if(hp.isTransactional())
	    conf.setTransactional(true);
	XmlContainer cont = hp.createContainer("resolveSchema.dbxml", mgr, conf);
	XmlDocument doc = mgr.createDocument();
	doc.setName("doc1.xml");
	String str = "<?xml version=\"1.0\"?><root xmlns=\"http://fake.uri."
	    + "schema\" xmlns:xsi=\"http://www.w3.org/2001/XML"
	    + "Schema-instance\" xsi:schemaLocation=\"http://"
	    + "fake.uri.schema root.xsd\"></root>";
	doc.setContent(str);
	try {
	    mgr.registerResolver(test);
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc);
	    }else
		cont.putDocument(doc);
	} catch (XmlException e) {
	    test.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	    cont.delete();
	    throw e;
	}

	//put the not valid xml 
	XmlDocument doc2 = mgr.createDocument();
	str = "<a xmlns=\"http://fake.uri.schema\" xmlns:xsi=\"http://"
	    + "www.w3.org/2001/XMLSchema-instance\" xsi:schemaL"
	    + "ocation=\"http://fake.uri.schema note.xsd\"><b/></a>";
	doc2.setName("fail.xml");
	doc2.setContent(str);
	try {
	    if(hp.isTransactional())
		cont.putDocument(txn, doc2);
	    else
		cont.putDocument(doc2);
	    fail("Fail in testResolveSchema()");
	} catch (XmlException e) {
	    assertNotNull(e.getMessage());
	    assertEquals(e.getErrorCode(), XmlException.INDEXER_PARSER_ERROR);
	}finally{
	    test.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	    cont.delete();
	}
    }

    /*
     * Test resolveModule & resolveModuleLocation
     * 
     */
    @Test
	public void testResolveModule() throws Throwable {
	TestResolver test = new TestResolver();
	mgr.registerResolver(test);
	String qs = "import module namespace test = 'http://test.module';\n\n test:function()";
	XmlQueryContext context = mgr.createQueryContext();

	XmlResults res = null;
	try{
	    res = mgr.query(qs, context);
	    XmlValue vl = res.next();
	    assertEquals(res.size(), 1);
	    assertEquals(vl.asString(), "module");
	}catch (XmlException e){
	    test.delete();
	    throw e;
	}finally{
	    if(res != null)res.delete();
	    res = null;
	}

	//not vaild query module
	qs = "import module namespace test = 'http://test.module';\n\n test:no()";
	try{
	    res = mgr.query(qs, context);
	    fail("Fail in testResolveModule()");
	}catch (XmlException e){
	    assertNotNull(e.getMessage());
	    assertEquals(e.getErrorCode(), XmlException.QUERY_PARSER_ERROR);
	}finally{
	    test.delete();
	    if(res != null)res.delete();
	    res = null;
	}
    }

    /*
     * Test resolveModuleLocation
     * 
     */
    @Test
	public void testResolveModuleLocation() throws Throwable {
	TestResolver test = new TestResolver();
	XmlManager mgr = new XmlManager();
	mgr.registerResolver(test);
	String qs = "import module namespace test = 'http://test.module';\n\n test:function()";
	XmlQueryContext context = mgr.createQueryContext();

	XmlResults res = null;
	try{
	    res = mgr.query(qs, context);
	    XmlValue vl = res.next();
	    assertEquals(res.size(), 1);
	    assertEquals(vl.asString(), "module");
	}catch (XmlException e){
	    test.delete();
	    if(res != null) res.delete();
	    mgr.delete();
	    throw e;
	}
	res.delete();
	res = null;

	//not vaild query module
	qs = "import module namespace test = 'http://test.module';\n\n test:no()";
	try{
	    res = mgr.query(qs, context);
	    res.delete();
	    test.delete();
	    mgr.delete();
	    fail("Fail in testResolveModule()");
	}catch (XmlException e){
	    assertNotNull(e.getMessage());
	    assertEquals(e.getErrorCode(), XmlException.QUERY_PARSER_ERROR);
	}
	XmlResults results = mgr.createResults();
	XmlTransaction txn = null;
	try{
	    test.resolveModuleLocation(txn, mgr, "", results);

	    assertEquals(results.size(), 1);
	    assertEquals(results.next().asString(), "http://dbxmltest.test/test");
	}catch(XmlException e){
	    throw e;
	}finally{
	    test.delete();
	    results.delete();
	    mgr.delete();
	}
    }

}
